@@ -131,7 +131,9 @@ class PagesIndexer(IndexerBase):
 			return # nothing to do
 
 		row = self._select(pagename)
-		assert row is not None
+		if row is None:
+			self.insert_page(pagename, filerow['id']) # May have been changed file type
+			row = self._select(pagename)
 
 		if row['source_file'] == filerow['id']:
 			file = self.layout.root.file(filerow['path'])
@@ -148,7 +150,8 @@ class PagesIndexer(IndexerBase):
 			return # nothing to do
 
 		row = self._select(pagename)
-		assert row is not None
+		if row is None:
+			return # not a source file after all (e.g. .txt attachment)
 
 		if row['source_file'] == filerow['id']:
 			if row['n_children'] > 0:
@@ -166,7 +169,7 @@ class PagesIndexer(IndexerBase):
 			else:
 				self.remove_page(pagename)
 		else:
-			raise NotImplemented # some conflict removed
+			pass # some conflict removed - or not a source file in the first place (.txt attachment)
 
 	def insert_page(self, pagename, file_id):
 		return self._insert_page(pagename, False, file_id)
@@ -386,6 +389,11 @@ class PagesViewInternal(object):
 			# By default ignore link placeholders to avoid circular
 			# dependencies between links and placeholders
 			assert href.rel == HREF_REL_FLOATING
+
+			# link to anchor on current page
+			if not href.parts():
+				return (start, start_id, relnames + href.parts())
+
 			anchor_key = natural_sort_key(href.parts()[0])
 
 			if relnames:
