@@ -1,5 +1,5 @@
 
-# Copyright 2008-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2020 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 
@@ -17,19 +17,19 @@ from functools import partial
 import zim.templates
 import zim.formats
 
-from zim.fs import File, Dir, SEP
+from zim.fs import File, Dir, SEP, adapt_from_newfs
 from zim.newfs import LocalFolder
 from zim.config import INIConfigFile, String, ConfigDefinitionByClass, Boolean, Choice
 from zim.errors import Error
 from zim.utils import natural_sort_key
 from zim.newfs.helpers import TrashNotSupportedError
 from zim.config import HierarchicDict
-from zim.parsing import link_type, is_win32_path_re
+from zim.parsing import link_type, is_win32_path_re, valid_interwiki_key
 from zim.signals import ConnectorMixin, SignalEmitter, SIGNAL_NORMAL
 
 from .operations import notebook_state, NOOP, SimpleAsyncOperation, ongoing_operation
 from .page import Path, Page, HRef, HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
-from .index import IndexNotFoundError, LINK_DIR_BACKWARD
+from .index import IndexNotFoundError, LINK_DIR_BACKWARD, ROOT_PATH
 
 DATA_FORMAT_VERSION = (0, 4)
 
@@ -53,9 +53,13 @@ class NotebookConfig(INIConfigFile):
 			('home', ConfigDefinitionByClass(Path('Home'))),
 			('icon', String(None)), # XXX should be file, but resolves relative
 			('document_root', String(None)), # XXX should be dir, but resolves relative
+			('short_links', Boolean(False)),
 			('shared', Boolean(True)),
 			('endofline', Choice(endofline, {'dos', 'unix'})),
 			('disable_trash', Boolean(False)),
+			('default_file_format', String('zim-wiki')),
+			('default_file_extension', String('.txt')),
+			('notebook_layout', String('files')),
 		))
 
 
@@ -83,8 +87,9 @@ def _resolve_relative_config(dir, config):
 
 def _iswritable(dir):
 	if os.name == 'nt':
-		# Test access - (iswritable turns out to be unreliable
-		# for folders on windows..)
+		# Test access - (iswritable turns out to be unreliable for folders on windows..)
+		if isinstance(dir, Dir):
+			dir = LocalFolder(dir.path) # only newfs supports cleanup=False
 		f = dir.file('.zim.tmp')
 		try:
 			f.write('Test')
@@ -126,14 +131,20 @@ class PageNotAllowedError(PageNotFoundError):
 			# T: description for PageNotAllowedError
 
 
+class PageNotAvailableError(PageNotFoundError):
+	_msg = _('Page not available: %s') # T: message for PageNotAvailableError
+	description = _('This page name cannot be used due to a conflicting file in the storage')
+			# T: description for PageNotAvailableError
+
+	def __init__(self, path, file):
+		PageError.__init__(self, path)
+		self.file = file
+
+
 class PageExistsError(Error):
 	_msg = _('Page already exists: %s') # T: message for PageExistsError
 
 
-class PageReadOnlyError(Error):
-	_msg = _('Can not modify page: %s') # T: error message for read-only pages
-
-
 class IndexNotUptodateError(Error):
 	pass # TODO description here?
 
@@ -228,6 +239,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		@param dir: a L{Dir} object
 		@returns: a L{Notebook} object
 		'''
+		dir = adapt_from_newfs(dir)
 		assert isinstance(dir, Dir)
 
 		nb = _NOTEBOOK_CACHE.get(dir.uri)
@@ -238,17 +250,26 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		from .layout import FilesLayout
 
 		config = NotebookConfig(dir.file('notebook.zim'))
-		endofline = config['Notebook']['endofline']
-		shared = config['Notebook']['shared']
 
-		subdir = dir.subdir('.zim')
-		if not shared and subdir.exists() and _iswritable(subdir):
-			cache_dir = subdir
-		else:
+		if config['Notebook']['shared']:
 			cache_dir = _cache_dir_for_dir(dir)
+		else:
+			cache_dir = dir.subdir('.zim')
+			cache_dir.touch()
+			if not (cache_dir.exists() and _iswritable(cache_dir)):
+				cache_dir = _cache_dir_for_dir(dir)
 
 		folder = LocalFolder(dir.path)
-		layout = FilesLayout(folder, endofline)
+		if config['Notebook']['notebook_layout'] == 'files':
+			layout = FilesLayout(
+				folder,
+				config['Notebook']['endofline'],
+				config['Notebook']['default_file_format'],
+				config['Notebook']['default_file_extension']
+			)
+		else:
+			raise ValueError('Unkonwn notebook layout: %s' % config['Notebook']['notebook_layout'])
+
 		cache_dir.touch() # must exist for index to work
 		index = Index(cache_dir.file('index.db').path, layout)
 
@@ -283,6 +304,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		self.name = None
 		self.icon = None
 		self.document_root = None
+		self.interwiki = None
 
 		if folder.watcher is None:
 			from zim.newfs.helpers import FileTreeWatcher
@@ -309,6 +331,21 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		self.connectto(self.properties, 'changed', self.on_properties_changed)
 		self.on_properties_changed(self.properties)
 
+	def __repr__(self):
+		return '<%s: %s>' % (self.__class__.__name__, self.name)
+
+	def _reload_pages_in_cache(self, path):
+		p = path.name
+		ns = path.name + ':'
+		for name, page in self._page_cache.items():
+			if name == p or name.startswith(ns):
+				if page.modified:
+					logger.error('Page with unsaved changes in cache while modifying notebook')
+				else:
+					page.reload_textbuffer()
+					# "page.haschildren" may also have changed, will be updated
+					# by signal handlers for index
+
 	@property
 	def uri(self):
 		'''Returns a file:// uri for this notebook that can be opened by zim'''
@@ -367,7 +404,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 	def on_properties_changed(self, properties):
 		dir = Dir(self.layout.root.path) # XXX
 
-		self.name = properties['name']
+		self.name = properties['name'] or self.folder.basename
 		icon, document_root = _resolve_relative_config(dir, properties)
 		if icon:
 			self.icon = icon.path # FIXME rewrite to use File object
@@ -375,6 +412,8 @@ class Notebook(ConnectorMixin, SignalEmitter):
 			self.icon = None
 		self.document_root = document_root
 
+		self.interwiki = valid_interwiki_key(properties['interwiki'] or self.name)
+
 	def suggest_link(self, source, word):
 		'''Suggest a link Path for 'word' or return None if no suggestion is
 		found. By default we do not do any suggestion but plugins can
@@ -404,16 +443,21 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		# As a special case, using an invalid page as the argument should
 		# return a valid page object.
 		assert isinstance(path, Path)
-		if path.name in self._page_cache \
-		and self._page_cache[path.name].valid:
+		if path.name in self._page_cache:
 			page = self._page_cache[path.name]
 			assert isinstance(page, Page)
-			page._check_source_etag()
+			page.check_source_changed()
 			return page
 		else:
 			file, folder = self.layout.map_page(path)
+			if file.exists() and not self.layout.is_source_file(file):
+				raise PageNotAvailableError(path, file)
+
 			folder = self.layout.get_attachments_folder(path)
-			page = Page(path, False, file, folder)
+			format = self.layout.get_format(file)
+			page = Page(path, False, file, folder, format)
+			if self.readonly:
+				page._readonly = True # XXX
 			try:
 				indexpath = self.pages.lookup_by_pagename(path)
 			except IndexNotFoundError:
@@ -444,33 +488,17 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		'''
 		i = 0
 		base = path.name
-		page = self.get_page(path)
-		while page.hascontent or page.haschildren:
-			i += 1
-			path = Path(base + ' %i' % i)
-			page = self.get_page(path)
-		return page
-
-	@notebook_state
-	def flush_page_cache(self, path):
-		'''Flush the cache used by L{get_page()}
-
-		After this method calling L{get_page()} for C{path} or any of
-		its children will return a fresh page object. Be aware that the
-		old Page objects may still be around but will be flagged as
-		invalid and can no longer be used in the API.
-
-		@param path: a L{Path} object
-		'''
-		names = [path.name]
-		ns = path.name + ':'
-		names.extend(k for k in list(self._page_cache.keys()) if k.startswith(ns))
-		for name in names:
-			if name in self._page_cache:
-				page = self._page_cache[name]
-				assert not page.modified, 'BUG: Flushing page with unsaved changes'
-				page.valid = False
-				del self._page_cache[name]
+		while True:
+			try:
+				page = self.get_page(path)
+			except PageNotAvailableError:
+				pass
+			else:
+				if not (page.hascontent or page.haschildren):
+					return page
+			finally:
+				i += 1
+				path = Path(base + ' %i' % i)
 
 	def get_home_page(self):
 		'''Returns a L{Page} object for the home page'''
@@ -484,18 +512,16 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		@emits: store-page before storing the page
 		@emits: stored-page on success
 		'''
-		assert page.valid, 'BUG: page object no longer valid'
 		logger.debug('Store page: %s', page)
 		self.emit('store-page', page)
 		page._store()
 		file, folder = self.layout.map_page(page)
 		self.index.update_file(file)
-		page.modified = False
+		page.set_modified(False)
 		self.emit('stored-page', page)
 
 	@notebook_state
 	def store_page_async(self, page, parsetree):
-		assert page.valid, 'BUG: page object no longer valid'
 		logger.debug('Store page in background: %s', page)
 		self.emit('store-page', page)
 		error = threading.Event()
@@ -529,7 +555,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 				# HACK: Checking modified state protects against race condition
 				# in async store. Works because pageview sets "page.modified"
 				# to a counter rather than a boolean
-				page.modified = False
+				page.set_modified(False)
 				self.emit('stored-page', page)
 
 	def wait_for_store_page_async(self):
@@ -577,7 +603,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		file, folder = self.layout.map_page(path)
 		if (file.exists() or folder.exists()):
 			self._move_file_and_folder(path, newpath)
-			self.flush_page_cache(path)
+			self._reload_pages_in_cache(path)
 			self.emit('moved-page', path, newpath)
 
 			if update_links:
@@ -613,7 +639,12 @@ class Notebook(ConnectorMixin, SignalEmitter):
 				pass # renaming on case-insensitive filesystem
 			elif newfile.exists() or newfolder.exists():
 				raise PageExistsError(newpath)
-		elif newfile.exists() or newfolder.exists():
+		elif newfile.exists():
+			if self.layout.is_source_file(newfile):
+				raise PageExistsError(newpath)
+			else:
+				raise PageNotAvailableError(newpath, newfile)
+		elif newfolder.exists():
 			raise PageExistsError(newpath)
 
 		# First move the dir - if it fails due to some file being locked
@@ -793,13 +824,28 @@ class Notebook(ConnectorMixin, SignalEmitter):
 	def _update_link_tag(self, elt, source, target, oldhref):
 		if oldhref.rel == HREF_REL_ABSOLUTE: # prefer to keep absolute links
 			newhref = HRef(HREF_REL_ABSOLUTE, target.name)
+		elif source == target and oldhref.anchor:
+			newhref = HRef(HREF_REL_FLOATING, '', oldhref.anchor)
 		else:
 			newhref = self.pages.create_link(source, target)
 
-		text = newhref.to_wiki_link()
+		newhref.anchor = oldhref.anchor
+
+		link = newhref.to_wiki_link()
+
 		if elt.gettext() == elt.get('href'):
-			elt[:] = [text]
-		elt.set('href', text)
+			elt[:] = [link]
+		elif elt.gettext() == oldhref.parts()[-1] and len(elt) == 1:
+			# We are using short links and the link text was short link
+			# and there were no sub-node (like bold text) that would be cancelled.
+			# Related to 'short_links' but not checking the property here.
+			short = newhref.parts()[-1]
+			if newhref.anchor:
+				short += '#' + newhref.anchor
+			elt[:] = [short]  # 'Journal:2020:01:20' -> '20'
+
+		elt.set('href', link)
+
 		return elt
 
 	@assert_index_uptodate
@@ -927,7 +973,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 		return re
 
 	def _deleted_page(self, path, update_links):
-		self.flush_page_cache(path)
+		self._reload_pages_in_cache(path)
 		path = Path(path.name)
 
 		if update_links:
@@ -1078,7 +1124,7 @@ class Notebook(ConnectorMixin, SignalEmitter):
 				parent = file.commonparent(attachments_dir)
 				uppath = attachments_dir.relpath(parent)
 				downpath = file.relpath(parent)
-				up = 1 + uppath.count('/')
+				up = 1 + uppath.replace('\\', '/').count('/')
 				return updir * up + downpath
 		else:
 			if document_root and notebook_root \
