@@ -140,6 +140,8 @@ class ToCPageViewExtension(PageViewExtension):
 		if not isinstance(self.tocwidget, widgetclass):
 			if isinstance(self.tocwidget, SidePaneToC):
 				self.remove_sidepane_widget(self.tocwidget)
+			elif self.tocwidget:
+				self.tocwidget.destroy()
 
 			self.tocwidget = widgetclass(self.pageview)
 
@@ -322,7 +324,8 @@ class ToCWidget(ConnectorMixin, Gtk.ScrolledWindow):
 		if tree is None:
 			model.clear()
 		else:
-			model.update(get_headings(tree, self.include_hr), self.show_h1)
+			if model is not None:
+				model.update(get_headings(tree, self.include_hr), self.show_h1)
 		self.emit('changed')
 
 	def on_heading_activated(self, treeview, path, column):
@@ -538,7 +541,7 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 			handler=DelayedCallback(10, self.update_size_and_position),
 				# Callback wrapper to prevent glitches for fast resizing of the window
 		)
-		self.connectto(self.tocwidget, 'changed', handler=self.update_size_and_position)
+		self.connectto(self.tocwidget, 'changed', handler=self.update_size_and_position_after_change)
 
 		self._event_box.show_all()
 
@@ -559,9 +562,13 @@ class FloatingToC(Gtk.VBox, ConnectorMixin):
 		)
 		self.update_size_and_position()
 
+	def update_size_and_position_after_change(self, *a):
+		self.tocwidget.treeview.expand_all()
+		self.update_size_and_position()
+
 	def update_size_and_position(self, *a):
 		model = self.tocwidget.treeview.get_model()
-		if model.is_empty:
+		if model is None or model.is_empty:
 			self.hide()
 			return
 		else:
